@@ -3,13 +3,15 @@
 #' @importFrom checkmate test_int test_logical test_choice test_string
 #' @importFrom dplyr n_distinct
 #' @keywords internal
-check_arguments <- function(mc, dotArgs) {
+check_args <- function(mc, dotArgs) {
   
-  caller <- as.character(sys.call(-1))[1]
-  pf <- parent.frame()
-  errmsg <- character()
+  caller      <- sub(".+::","",as.character(sys.call(-1))[1])
+  pf          <- parent.frame()
+  errmsg      <- character()
+  caller_orig <- caller
   
   if (caller == "FUN") {
+    pf$flag_by <- TRUE
     # When stby() was called, deduce caller from formals
     if ("order" %in% names(pf))
       caller <- "freq"
@@ -19,6 +21,8 @@ check_arguments <- function(mc, dotArgs) {
       caller <- "ctable"
     else if ("graph.col" %in% names(pf))
       caller <- "dfSummary"
+  } else {
+    pf$flag_by <- FALSE
   }
   
   # Deprecated arguments -------------------------------------------------------
@@ -86,7 +90,7 @@ check_arguments <- function(mc, dotArgs) {
   if ("headings" %in% names(mc) &&
       !isTRUE(test_logical(pf$headings, 
                            len = 1, any.missing = FALSE))) {
-    errmsg %+=% "'headings' must be either be TRUE or FALSE"
+    errmsg %+=% "'headings' must be either TRUE or FALSE"
   }
   
   # freq-specific arguments ----------------------------------------------------
@@ -180,7 +184,8 @@ check_arguments <- function(mc, dotArgs) {
     if (!identical(pf$weights, NA)) {
       if (is.null(pf$weights)) {
         errmsg %+=% "weights vector not found"
-      } else if (length(pf$weights) != nrow(as.data.frame(pf$x))) {
+      } else if (caller_orig != "FUN" && 
+                 length(pf$weights) != nrow(as.data.frame(pf$x))) {
         errmsg %+=% "weights vector must have same length as 'x'"      
       }
     }
@@ -241,7 +246,7 @@ check_arguments <- function(mc, dotArgs) {
     if (!identical(pf$weights, NA)) {
       if (is.null(pf$weights)) {
         errmsg %+=% "weights vector not found"
-      } else if (length(pf$weights) != nrow(pf$x.df)) {
+      } else if (caller_orig != "FUN" && (length(pf$weights) != nrow(pf$x.df))) {
         errmsg %+=% "weights vector must have same length as 'x'"      
       }
     }
@@ -318,16 +323,38 @@ check_arguments <- function(mc, dotArgs) {
     }
     
     if ("tmp.img.dir" %in% names(mc) && !is.na(pf$tmp.img.dir) &&
-        isTRUE(.st_env$noX11)) {
-      message("'tmp.img.dir' will be ignored since system has no X11 ",
-              "capabilities")
+        isFALSE(st_options("use.x11"))) {
+      message("'tmp.img.dir' will be ignored since use.x11 option is set to ",
+              "FALSE")
     }
   }
   return(errmsg)
 }
 
+check_args_tb <- function(mc) {
+  
+  pf <- parent.frame()
+  errmsg <- character()
 
-check_arguments_print <- function(mc) {
+  if ("order" %in% names(mc) &&
+      !isTRUE(test_choice(pf$order, c(1, 2, 3)))) {
+    errmsg %+=% "'order' must be one of 1, 2, or 3"
+  }
+  
+  if ("na.rm" %in% names(mc) && 
+      !isTRUE(test_logical(pf$na.rm, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'na.rm' must be either TRUE or FALSE"
+  }
+
+  if ("drop.val.col" %in% names(mc) && 
+      !isTRUE(test_logical(pf$na.rm, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'na.rm' must be either TRUE or FALSE"
+  }
+  
+  return(errmsg)
+}
+
+check_args_print <- function(mc) {
   
   pf <- parent.frame()
   errmsg <- character()
@@ -406,8 +433,8 @@ check_arguments_print <- function(mc) {
     errmsg %+=% "'custom.css' option does not apply to method 'pander'"
   }
   
-  # Set plain.ascii to false and adjust style when file name ends with .md
-  if (grepl("\\.md$", pf$file, ignore.case = TRUE, perl = TRUE) 
+  # Set plain.ascii to false and adjust style when file extension is .md or .Rmd
+  if (grepl("\\.r?md$", pf$file, ignore.case = TRUE, perl = TRUE) 
       && !"style" %in% names(pf$dotArgs)) {
     if (isTRUE(attr(pf$x, "format_info")$plain.ascii) && 
         !"plain.ascii" %in% names(pf$dotArgs)) {
@@ -452,39 +479,43 @@ check_arguments_print <- function(mc) {
  return(errmsg)
 }
   
-# check_arguments_st_options ---------------------------------------------------
+# check_args_st_options ---------------------------------------------------
 #' @importFrom checkmate test_int test_logical test_choice 
 #' test_file_exists test_character
-check_arguments_st_options <- function(mc) {
+check_args_st_options <- function(mc) {
   
   pf <- parent.frame()
   errmsg <- character()
   
+  if ("omit.headings" %in% names(mc)) {
+    errmsg %+=% "'omit.headings' is deprecated; use 'headings' instead"
+  }
+  
   if ("style" %in% names(mc)) {
     if (!isTRUE(test_choice(pf$style, 
                             c("simple", "grid", "rmarkdown")))) {
       errmsg %+=% paste("'style' must be one of 'simple', 'grid', or 'markdown';",
-                     "See documentation for details")
+                        "See documentation for details")
     }
   }
-
-  if ("round.digits" %in% names(mc) && !isTRUE(test_int(pf$round.digits))) {
-    errmsg %+=% "'round.digits' must be a whole number"
-  }
   
   if ("plain.ascii" %in% names(mc) && 
       !isTRUE(test_logical(pf$plain.ascii, len = 1, any.missing = FALSE))) {
     errmsg %+=% "'plain.ascii' must be either TRUE or FALSE"
   }
   
+  if ("round.digits" %in% names(mc) && !isTRUE(test_int(pf$round.digits))) {
+    errmsg %+=% "'round.digits' must be a whole number"
+  }
+  
   if ("headings" %in% names(mc) &&
       !isTRUE(test_logical(pf$headings, len = 1, any.missing = FALSE))) {
-    errmsg %+=% "'headings' must be either be TRUE or FALSE"
+    errmsg %+=% "'headings' must be either TRUE or FALSE"
   }
   
   if ("footnote" %in% names(mc) &&
       !isTRUE(test_character(pf$footnote)) && !is.na(pf$footnote)) {
-    errmsg %+=% "'footnote' must be either be a string or NA"
+    errmsg %+=% "'footnote' must be either a string or NA"
   }
   
   if ("display.labels" %in% names(mc) &&
@@ -509,6 +540,11 @@ check_arguments_st_options <- function(mc) {
     errmsg %+=% "'escape.pipe' must be either TRUE or FALSE"
   }
   
+  if ("freq.cumul" %in% names(mc) &&
+      !isTRUE(test_logical(pf$freq.cumul, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'freq.cumul' must be either TRUE or FALSE"
+  }
+  
   if ("freq.totals" %in% names(mc) &&
       !isTRUE(test_logical(pf$freq.totals, len = 1, any.missing = FALSE))) {
     errmsg %+=% "'freq.totals' must be either TRUE or FALSE"
@@ -518,12 +554,17 @@ check_arguments_st_options <- function(mc) {
       !isTRUE(test_logical(pf$freq.report.nas, len = 1, any.missing = FALSE))) {
     errmsg %+=% "'freq.report.nas' must be either TRUE or FALSE"
   }
-
+  
   if ("freq.ignore.threshold" %in% names(mc) &&
       !isTRUE(test_int(pf$freq.ignore.threshold, lower = 0))) {
     errmsg %+=% "'freq.ignore.threshold' must be an integer greater than 0"
   }
-
+  
+  if ("freq.silent" %in% names(mc) &&
+      !isTRUE(test_logical(pf$freq.silent, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'freq.silent' must be either TRUE or FALSE"
+  }
+  
   if ("ctable.prop" %in% names(mc) &&
       !isTRUE(test_choice(pf$ctable.prop, c("r", "c", "t", "n")))) {
     errmsg %+=% "'ctable.prop' must be one of \"r\", \"c\", \"t\", or \"n\""
@@ -534,10 +575,6 @@ check_arguments_st_options <- function(mc) {
     errmsg %+=% "'ctable.totals' must be either TRUE or FALSE"
   }
   
-  if ("omit.headings" %in% names(mc)) {
-    errmsg %+=% "'omit.headings' is deprecated; use 'headings' instead"
-  }
-  
   if ("descr_stats" %in% names(mc)) {
     valid_stats <- c("mean", "sd", "min", "q1", "med", "q3","max", "mad", 
                      "iqr", "cv", "skewness", "se.skewness", "kurtosis", 
@@ -559,12 +596,17 @@ check_arguments_st_options <- function(mc) {
     errmsg %+=% "'descr.transpose' must be either TRUE or FALSE"
   }
   
-  if ("freq.totals" %in% names(mc) &&
-      !isTRUE(test_logical(pf$freq.totals, 
-                           len = 1, any.missing = FALSE))) {
-    errmsg %+=% "'freq.totals' must be either TRUE or FALSE"
+  if ("descr.silent" %in% names(mc) &&
+      !isTRUE(test_logical(pf$descr.silent, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'descr.silent' must be either TRUE or FALSE"
   }
-  
+
+  if ("dfSummary.style" %in% names(mc) &&
+      !isTRUE(test_choice(pf$dfSummary.style, 
+                          c("grid", "multiline")))) {
+    errmsg %+=% "'dfSummary.style' must be either 'grid' or 'multiline'"
+  }
+    
   if ("dfSummary.varnumbers" %in% names(mc) &&
       !isTRUE(test_logical(pf$dfSummary.varnumbers, 
                            len = 1, any.missing = FALSE))) {
@@ -576,11 +618,13 @@ check_arguments_st_options <- function(mc) {
                            len = 1, any.missing = FALSE))) {
     errmsg %+=% "'dfSummary.labels.col' must be either TRUE or FALSE"
   }
+  
   if ("dfSummary.valid.col" %in% names(mc) &&
       !isTRUE(test_logical(pf$dfSummary.valid.col,
                            len = 1, any.missing = FALSE))) {
     errmsg %+=% "'dfSummary.valid.col' must be either TRUE or FALSE"
   }
+  
   if ("dfSummary.na.col" %in% names(mc) &&
       !isTRUE(test_logical(pf$dfSummary.na.col, 
                            len = 1, any.missing = FALSE))) {
@@ -597,18 +641,35 @@ check_arguments_st_options <- function(mc) {
       pf$dfSummary.graph.magnif <= 0) {
     errmsg %+=% "'dfSummary.graph.magnif' must be > 0"
   }
-  
-  if ("lang" %in% names(mc) && !pf$lang %in% 
-      c(rownames(.translations), "custom")) {
-    errmsg %+=% paste0("'lang' can take the following values only: ",
-                       paste(rownames(.translations), collapse = ", "))
-  }
+
+  if ("dfSummary.silent" %in% names(mc) &&
+      !isTRUE(test_logical(pf$dfSummary.silent,
+                           len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'dfSummary.silent' must be either TRUE or FALSE"
+  }  
   
   if ("tmp.img.dir" %in% names(mc) &&
       (!isTRUE(test_character(pf$tmp.img.dir, min.chars = 1, len = 1)) ||
        nchar(pf$tmp.img.dir) > 5)) {
     errmsg %+=% "'tmp.img.dir' must have at least 1 and at most 5 characters"
   }
+
+  if ("subtitle.emphasis" %in% names(mc) &&
+      !isTRUE(test_logical(pf$subtitle.emphasis, 
+                           len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'subtitle.emphasis' must be either TRUE or FALSE"
+  }
+    
+  if ("lang" %in% names(mc) && !pf$lang %in% 
+      c(rownames(.translations), "custom")) {
+    errmsg %+=% paste0("'lang' can take the following values only: ",
+                       paste(rownames(.translations), collapse = ", "))
+  }
+
+  if ("use.x11" %in% names(mc) &&
+      !isTRUE(test_logical(pf$use.x11, len = 1, any.missing = FALSE))) {
+    errmsg %+=% "'use.x11' must be either TRUE or FALSE"
+  }
   
   return(errmsg)
 }
