@@ -161,6 +161,10 @@ descr <- function(x,
           attr(outlist[[g]], "data_info")$Variable <- parse_info$var_name
         } else if (exists("varname")) {
           attr(outlist[[g]], "data_info")$Variable <- varname
+          if (identical(colnames(outlist[[g]]), "value"))
+            colnames(outlist[[g]]) <- varname
+          if (identical(rownames(outlist[[g]]), "value"))
+            rownames(outlist[[g]]) <- varname
         }
         if (!is.null(parse_info$var_label))
           attr(outlist[[g]], "data_info")$Variable.label <- parse_info$var_label
@@ -185,8 +189,7 @@ descr <- function(x,
     return(outlist)
   }
   
-  # When var is provided, we all other variables
-  # variables in the analysis
+  # When var is provided, discard all other variables
   if (is.data.frame(x) && ncol(x) > 1 && "var" %in% names(match.call())) {
     
     # var might contain a function call -- such as df %>% descr(na.omit(var1))
@@ -199,8 +202,10 @@ descr <- function(x,
       varname <- deparse(substitute(var))
     }
   } else {
-    x_obj    <- x
-    varname  <- colnames(x)
+    x_obj <- x
+    if (!is.null(colnames(x))) {
+      varname  <- colnames(x)
+    }
   }
   
   # Validate arguments -------------------------------------------------------
@@ -230,7 +235,7 @@ descr <- function(x,
                      "a data.table; attempted conversion to tibble failed")
   }
   
-  errmsg <- c(errmsg, check_arguments(match.call(), list(...)))
+  errmsg <- c(errmsg, check_args(match.call(), list(...)))
   
   valid_stats <- list(
     no_wgts = c("mean", "sd", "min", "q1", "med", "q3","max", "mad", 
@@ -347,7 +352,7 @@ descr <- function(x,
       rownames(output) <- parse_info$var_name
     }
 
-    # Calculate additionnal stats if needed
+    # Calculate additional stats if needed
     if ("cv" %in% stats) {
       output$cv <- output$sd / output$mean
     }
@@ -523,8 +528,7 @@ descr <- function(x,
                                 sub(pattern = paste0(parse_info$df_name, "$"), 
                                     replacement = "", x = weights_string, 
                                     fixed = TRUE)),
-      by_var           = ifelse("by_var" %in% names(parse_info),
-                                parse_info$by_var, NA),
+      by_var           = NA,
       Group            = ifelse("by_group" %in% names(parse_info),
                                 parse_info$by_group, NA),
       by_first         = ifelse("by_group" %in% names(parse_info), 
@@ -534,6 +538,10 @@ descr <- function(x,
       transposed       = transpose,
       N.Obs            = nrow(x.df))
   
+  if ("by_var" %in% names(parse_info)) {
+    data_info$by_var <- parse_info$by_var
+  }
+  
   attr(output, "data_info") <- data_info[!is.na(data_info)]
   
   attr(output, "format_info") <- list(style          = style,
