@@ -131,6 +131,9 @@ freq <- function(x,
                  rescale.weights = FALSE,
                  ...) {
 
+  # Initialize flag_by variable that will be set in the args validation function
+  flag_by <- logical()
+  
   # handle objects of class "grouped_df" (dplyr::group_by)
   if (inherits(x, "grouped_df")) {
     
@@ -169,8 +172,17 @@ freq <- function(x,
     gr_inds  <- attr(x, "groups")$.rows
     #ana_var  <- ifelse("var" %in% names(match.call()), var, 
     #                   setdiff(colnames(x), group_vars(x)))
+
+    if ("weights" %in% names(match.call())) {
+      weights_str <- deparse(substitute(weights))
+      weights_all <- parent.frame()$.[[weights_str]]
+    }
     
     for (g in seq_along(gr_ks)) {
+      if ("weights" %in% names(match.call())) {
+        weights <- weights_all[gr_inds[[g]]]
+      }
+      
       outlist[[g]] <- freq(x               = var_obj[gr_inds[[g]]],
                            round.digits    = round.digits,
                            order           = order,
@@ -193,6 +205,10 @@ freq <- function(x,
         attr(outlist[[g]], "data_info")$Data.frame <- parse_info$df_name
       }
       
+      if (exists("weights_str")) {
+        attr(outlist[[g]], "data_info")$Weights <- weights_str
+      }
+      
       if (!is.na(label(x))) {
         attr(outlist[[g]], "data_info")$Data.frame.label <- label(x)
       }
@@ -313,7 +329,7 @@ freq <- function(x,
     }
     
     order_sign <- "+"
-    errmsg <- c(errmsg, check_arguments(match.call(), list(...)))
+    errmsg <- c(errmsg, check_args(match.call(), list(...)))
     
     if (length(errmsg) > 0) {
       stop(paste(errmsg, collapse = "\n  "))
@@ -329,7 +345,9 @@ freq <- function(x,
     
     # Replace NaN's by NA's (This simplifies matters a lot)
     if (NaN %in% x)  {
-      message(paste(sum(is.nan(x)), "NaN value(s) converted to NA\n"))
+      if (isFALSE(st_options("freq.silent"))) {
+        message(paste(sum(is.nan(x)), "NaN value(s) converted to NA\n"))
+      }
       x[is.nan(x)] <- NA
     }
     
@@ -363,6 +381,12 @@ freq <- function(x,
       # Weights are used
       weights_string <- deparse(substitute(weights))
       
+      # Subset weights when called from by()/stby() to match current data subset
+      if (isTRUE(flag_by)) {
+        pf <- parent.frame(2)
+        weights <- weights[pf$X[[pf$i]]]
+      }
+      
       if (sum(is.na(weights)) > 0) {
         warning("missing values on weight variable have been detected and ",
                 "were treated as zeroes")
@@ -474,7 +498,9 @@ freq <- function(x,
     
     if (is.factor(x) && "(Missing)" %in% levels(x)
         && sum(is.na(x)) == 0 && isTRUE(report.nas)) {
-      message("explicit NA's detected - setting 'report.nas' to FALSE")
+      if (isFALSE(st_options("freq.silent"))) {
+        message("explicit NA's detected - temporarily setting 'report.nas' to FALSE")
+      }
       report.nas <- FALSE
       # hack the fn_call attribute to prevent print method from overriding it 
       tmp_args <- append(as.list(fn_call)[-1], list(report.nas = FALSE))
@@ -561,4 +587,4 @@ freq <- function(x,
     return(output)
     
   }
-}
\ No newline at end of file
+}
